@@ -14,7 +14,9 @@ from __future__ import absolute_import
 
 import os
 import re
+import posixpath
 import traceback
+import pkg_resources
 from collections import namedtuple
 from docutils import nodes
 from sphinx import addnodes
@@ -22,9 +24,10 @@ from sphinx.util.osutil import ensuredir
 
 import nwdiag.utils.rst.nodes
 import nwdiag.utils.rst.directives
-from blockdiag.utils.bootstrap import detectfont
+from blockdiag.utils.bootstrap import detectfont, Application
 from blockdiag.utils.compat import u, string_types
 from blockdiag.utils.fontmap import FontMap
+from blockdiag.utils.rst.directives import with_blockdiag
 
 # fontconfig; it will be initialized on `builder-inited` event.
 fontmap = None
@@ -51,15 +54,20 @@ class nwdiag_node(nwdiag.utils.rst.nodes.nwdiag):
         options = dict(antialias=builder.config.nwdiag_antialias,
                        fontpath=builder.config.nwdiag_fontpath,
                        fontmap=builder.config.nwdiag_fontmap,
-                       format=image_format)
-        outputdir = getattr(builder, 'imgpath', builder.outdir)
-        return os.path.join(outputdir, self.get_path(**options))
+                       format=image_format,
+                       transparency=builder.config.nwdiag_transparency)
+        if hasattr(builder, 'imgpath'):  # Sphinx (<= 1.2.x) or HTML writer
+            outputdir = builder.imgpath
+        else:
+            outputdir = ''
+        return posixpath.join(outputdir, self.get_path(**options))
 
     def get_abspath(self, image_format, builder):
         options = dict(antialias=builder.config.nwdiag_antialias,
                        fontpath=builder.config.nwdiag_fontpath,
                        fontmap=builder.config.nwdiag_fontmap,
-                       format=image_format)
+                       format=image_format,
+                       transparency=builder.config.nwdiag_transparency)
 
         if hasattr(builder, 'imagedir'):  # Sphinx (>= 1.3.x)
             outputdir = os.path.join(builder.outdir, builder.imagedir)
@@ -88,6 +96,8 @@ def resolve_reference(builder, href):
     matched = pattern.search(href)
     if matched is None:
         return href
+    elif not hasattr(builder, 'current_docname'):  # ex. latex builder
+        return matched.group(1)
     else:
         refid = matched.group(1)
         domain = builder.env.domains['std']
@@ -188,6 +198,7 @@ def html_render_png(self, node):
     self.body.append(self.starttag(node, 'img', '', empty=True, **img_attr))
 
 
+@with_blockdiag
 def html_visit_nwdiag(self, node):
     try:
         image_format = get_image_format_for(self.builder)
@@ -217,7 +228,7 @@ def html_depart_nwdiag(self, node):
 
 
 def get_image_format_for(builder):
-    if builder.format == 'html':
+    if builder.format in ('html', 'slides'):
         image_format = builder.config.nwdiag_html_image_format.upper()
     elif builder.format == 'latex':
         if builder.config.nwdiag_tex_image_format:
@@ -267,7 +278,7 @@ def on_builder_inited(self):
 
 
 def on_doctree_resolved(self, doctree, docname):
-    if self.builder.format == 'html':
+    if self.builder.format in ('html', 'slides'):
         return
 
     try:
@@ -284,14 +295,15 @@ def on_doctree_resolved(self, doctree, docname):
 
     for node in doctree.traverse(nwdiag_node):
         try:
-            relfn = node.get_relpath(image_format, self.builder)
-            image = node.to_drawer(image_format, self.builder)
-            if not os.path.isfile(image.filename):
-                image.draw()
-                image.save()
+            with Application():
+                relfn = node.get_relpath(image_format, self.builder)
+                image = node.to_drawer(image_format, self.builder)
+                if not os.path.isfile(image.filename):
+                    image.draw()
+                    image.save()
 
-            image = nodes.image(uri=image.filename, candidates={'*': relfn}, **node['options'])
-            node.parent.replace(node, image)
+                image = nodes.image(uri=relfn, candidates={'*': relfn}, **node['options'])
+                node.parent.replace(node, image)
         except Exception as exc:
             if self.builder.config.nwdiag_debug:
                 traceback.print_exc()
@@ -314,3 +326,9 @@ def setup(app):
     app.add_config_value('nwdiag_latex_image_format', 'PNG', 'html')
     app.connect("builder-inited", on_builder_inited)
     app.connect("doctree-resolved", on_doctree_resolved)
+
+    return {
+        'version': pkg_resources.require('nwdiag')[0].version,
+        'parallel_read_safe': True,
+        'parallel_write_safe': True,
+    }
